#!/bin/bash

# USB Gadget for TTRU3 hardware
set -e
#set -x

# command line parameters
command="$1" # "up" or "down"

g="/sys/kernel/config/usb_gadget/ttru3-gadget"

ev3_usb_massstorage_up() {
# Mass Storage device
###
mkdir ${g}/functions/mass_storage.usb0
echo 1 > ${g}/functions/mass_storage.usb0/stall
echo 1 > ${g}/functions/mass_storage.usb0/lun.0/cdrom
echo 1 > ${g}/functions/mass_storage.usb0/lun.0/ro
echo 0 > ${g}/functions/mass_storage.usb0/lun.0/nofua
#echo 0 > ${g}/functions/mass_storage.usb0/lun.0/removable
echo "${USBDISK}" > ${g}/functions/mass_storage.usb0/lun.0/file || true
ln -s ${g}/functions/mass_storage.usb0 ${g}/configs/c.1/
###
}


ev3_usb_up() {
sleep 1
modprobe libcomposite
sleep 1

usb_ver="0x0200" # USB 2.0
vid="0x1d6b"
pid="0x0104"
device="0x0101" # this should be incremented any time there are breaking changes
# to this script so that the host OS sees it as a new device and
# re-enumerates everything rather than relying on cached values
mfg="Megger"
prod="TTRU3"

readonly USBDISK="/usr/local/share/megger/TTRU3.iso"

# Read mac address from eeprom
#serial="$(grep Serial /proc/cpuinfo | sed 's/Serial\s*: 0000\(\w*\)/\1/')"
serial="5a:77:1e:1f:8e:9e"
attr="0xC0" # Self powered
pwr="1" # 2mA
cfg1="RNDIS"
# add colons for MAC address format
mac="$(echo ${serial} | sed 's/\(\w\w\)/:\1/g' | cut -b 2-)"
# Change the first number for each MAC address - the second digit of 2 indicates
# that these are "locally assigned (b2=1), unicast (b1=0)" addresses. This is
# so that they don't conflict with any existing vendors. Care should be taken
# not to change these two bits.
dev_mac2="22$(echo ${mac} | cut -b 3-)"
host_mac2="32$(echo ${mac} | cut -b 3-)"
ms_vendor_code="0xcd" # Microsoft
ms_qw_sign="MSFT100" # also Microsoft (if you couldn't tell)
ms_compat_id="RNDIS" # matches Windows RNDIS Drivers
ms_subcompat_id="5162001" # matches Windows RNDIS 6.0 Driver

#    tmpdir=$(mktemp -d)
#    mount /dev/mmcblk0p1 $tmpdir
#    if [ -f $tmpdir/ev3dev.txt ]; then
#        cdc_only=$(grep -i "^cdc_only=\(true\|yes\|1\)" $tmpdir/ev3dev.txt | cut -b 10-)
#    fi
#    umount $tmpdir

if [ -d ${g} ]; then
if [ "$(cat ${g}/UDC)" != "" ]; then
echo "Gadget is already up."
exit 1
fi
echo "Cleaning up old directory..."
ev3_usb_down
fi
echo "Setting up gadget..."

# Create a new gadget

mkdir ${g}
echo "${usb_ver}" > ${g}/bcdUSB
echo 0xEF > ${g}/bDeviceClass
echo 0x02 > ${g}/bDeviceSubClass
echo 0x01 > ${g}/bDeviceProtocol
echo "${vid}" > ${g}/idVendor
echo "${pid}" > ${g}/idProduct
echo "${device}" > ${g}/bcdDevice
mkdir ${g}/strings/0x409
echo "${mfg}" > ${g}/strings/0x409/manufacturer
echo "${prod}" > ${g}/strings/0x409/product
echo "${serial}" > ${g}/strings/0x409/serialnumber

echo "serial: ${serial}"

# config for RNDIS

mkdir ${g}/configs/c.1
echo "${attr}" > ${g}/configs/c.1/bmAttributes
echo "${pwr}" > ${g}/configs/c.1/MaxPower
mkdir ${g}/configs/c.1/strings/0x409
echo "${cfg1}" > ${g}/configs/c.1/strings/0x409/configuration

# On Windows 7 and later, the RNDIS 5.1 driver would be used by default,
# but it does not work very well. The RNDIS 6.0 driver works better. In
# order to get this driver to load automatically, we have to use a
# Microsoft-specific extension of USB.

echo "1" > ${g}/os_desc/use
echo "${ms_vendor_code}" > ${g}/os_desc/b_vendor_code
echo "${ms_qw_sign}" > ${g}/os_desc/qw_sign

# Create the RNDIS function, including the Microsoft-specific bits
###
mkdir ${g}/functions/rndis.usb0
echo "${dev_mac2}" > ${g}/functions/rndis.usb0/dev_addr
echo "${host_mac2}" > ${g}/functions/rndis.usb0/host_addr
echo "${ms_compat_id}" > ${g}/functions/rndis.usb0/os_desc/interface.rndis/compatible_id
echo "${ms_subcompat_id}" > ${g}/functions/rndis.usb0/os_desc/interface.rndis/sub_compatible_id
ln -s ${g}/functions/rndis.usb0 ${g}/configs/c.1
###

# Serial device
###
#mkdir ${g}/functions/acm.usb0
#ln -s ${g}/functions/acm.usb0 ${g}/configs/c.1/
###

# Mass Storage device
if [ ! -f "/var/ttru3/DISABLE_MASS_STORAGE" ]; then
ev3_usb_massstorage_up
fi

# Link everything up and bind the USB device

ln -s ${g}/configs/c.1 ${g}/os_desc

echo "Binding USB Device Controller"
echo `ls /sys/class/udc` > ${g}/UDC

ifconfig
echo "OK"
}

ev3_usb_massstorage_down() {
rm -f ${g}/configs/c.1/mass_storage.usb0
[ -d ${g}/functions/mass_storage.usb0 ] && rmdir ${g}/functions/mass_storage.usb0 || true
}

ev3_usb_down() {

ifconfig usb0 down || true

if [ ! -d ${g} ]; then
echo "Gadget is already down."
exit 1
fi
echo "Taking down gadget..."

# Have to unlink and remove directories in reverse order.
# Checks allow to finish takedown after error.

if [ "$(cat ${g}/UDC)" != "" ]; then
echo "" > ${g}/UDC
fi
rm -f ${g}/os_desc/c.1
rm -f ${g}/configs/c.1/acm.usb0
rm -f ${g}/configs/c.1/rndis.usb0
ev3_usb_massstorage_down
[ -d ${g}/functions/rndis.usb0 ] && rmdir ${g}/functions/rndis.usb0
[ -d ${g}/configs/c.1/strings/0x409 ] && rmdir ${g}/configs/c.1/strings/0x409
[ -d ${g}/configs/c.1 ] && rmdir ${g}/configs/c.1
[ -d ${g}/strings/0x409 ] && rmdir ${g}/strings/0x409
rmdir ${g}

echo "Done."
}

case ${command} in

up)
ev3_usb_up
;;
up_massstorage)
ev3_usb_massstorage_up
;;
down)
ev3_usb_down
;;
down_massstorage)
ev3_usb_massstorage_down
;;
*)
echo "Usage: ttr-usb-gadget.sh up|down|down_massstorage|down_massstorage"
exit 1
;;
esac
